#!/usr/bin/perl
#
# CombineTrees
# 
# Copyright (C) 2006  Akifumi S. Tanabe
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#use strict;

my $mode = 1;
my $outtree;
my @intree;
my $numtree = 0;

# Main routine

my $buildno = '20061101';

print << "_END";
CombineTrees build $buildno
======================

Copyright (C) 2006  Akifumi S. Tanabe

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

_END

# parse options
$outtree = pop(@ARGV);
if ($outtree =~ /(?:\/|\\)/) {
	&errorMessage();
}
foreach (@ARGV) {
	if (/^-+(?:help|h)/i) {
		&helpMessage();
	}
	elsif (/^-+(?:mode|m)=(.+)/i) {
		if ($1 =~ /^Average$/i) {
			$mode = 1;
		} elsif ($1 =~ /^Sum$/i) {
			$mode = 0;
		} else {
			&errorMessage('Unknown option.');
		}
	}
	elsif ($_ !~ /(?:\/|\\)/) {
		if (-e $_) {
			push(@intree, $_);
		} else {
			&errorMessage('Input tree file does not exist.');
		}
	} else {
		&errorMessage('Unknown option.');
	}
}

# count trees
unless (open(INTREE, "< $intree[0]")) {
	&errorMessage();
}
while (<INTREE>) {
	if (/\(.+\);/) {
		$numtree ++;
	}
}
close(INTREE);
#print("$numtree\n");

# make outtree
unless (open(OUTTREE, "> $outtree")) {
	&errorMessage();
}

for (my $treeno = 0; $treeno < $numtree; $treeno ++) {
	my %otu;
	my %clade;
	my @clade;
	for (my $fileno = 0; $fileno < @intree; $fileno ++) {
		# read branch lengths
		unless (open(INTREE, "< $intree[$fileno]")) {
			&errorMessage('Cannot open input tree.');
		}
		my $k = 0;
		while (<INTREE>) {
#			print;
			if (/\((.+)\);/) {
				my $tree = $1;
				$k ++;
				if ($k == $treeno + 1) {
					while ($tree =~ s/([^,:\(\)]+):(\d+\.?\d*)(e\D?\d+)?/$1/) {
						my $temp1 = $2;
						if ($3 =~ /e(\D)?0*(\d+)/) {
							my $temp2 = $2;
							if ($1 eq '-') {
								$temp1 *= 10 ** ((-1) * $temp2);
							} else {
								$temp1 *= 10 ** $temp2;
							}
						}
						$otu{$1} += $temp1;
#						print("$temp1\n");
					}
					my $num = 0;
					while ($tree =~ s/\(([^:\(\)]+)\):(\d+\.?\d*)(e\D?\d+)?/<$num>$1<\/$num>/) {
						my $temp1 = $2;
						if ($3 =~ /e(\D)?0*(\d+)/) {
							my $temp2 = $2;
							if ($1 eq '-') {
								$temp1 *= 10 ** ((-1) * $temp2);
							} else {
								$temp1 *= 10 ** $temp2;
							}
						}
						$clade{$num} += $temp1;
#						print("$temp1\n");
						$num++;
					}
#					print("$fileno:$treeno:$tree\n");
					if ($fileno == 0) {
						$tree = "<$num>" . $tree . "</$num>";
#						print("$tree\n");
						$num = 0;
						while ($tree =~ s/<(\d+)>([^<>]+)<\/\1>/\[$1\]/) {
#							print("$tree\n");
							@{$clade[$num]} = split (/,/, $2);
#							print("@{$clade[$num]}\n");
							$num++;
						}
					}
					last;
				}
			}
		}
		close(INTREE);
	}
	if ($mode) {
		foreach (keys(%otu)) {
			$otu{$_} /= @intree;
#			print("$otu{$_}\n");
		}
		foreach (keys(%clade)) {
			$clade{$_} /= @intree;
#			print("$clade{$_}\n");
		}
	}
	# output tree
	my $outputtree = '(' . join(',', @{$clade[@clade - 1]}) . ')';
#	print("$outputtree\n");
	for (my $cladeno = @clade - 2; $cladeno >= 0; $cladeno --) {
#		print("$cladeno\n");
		if ($outputtree =~ /\[$cladeno\]/) {
			my $clade = '(' . join(',', @{$clade[$cladeno]}) . '):' . $clade{$cladeno};
#			print("$clade\n");
			$outputtree =~ s/\[$cladeno\]/$clade/;
		} else {
			&errorMessage();
		}
	}
	foreach (keys(%otu)) {
		$outputtree =~ s/(\(|,)$_(,|\))/$1$_:$otu{$_}$2/;
	}
	print(OUTTREE "$outputtree;\n");
}

close(OUTTREE);

exit(0);

sub errorMessage {
	print("ERROR!: @_\n");
	print("If you want to read help message, run this script with \"--help\" option.\n");
	exit(1);
}

sub helpMessage {
	print << "_END";
Usage
=====

perl combinetree.pl [-m] intree1 intree2 .. intreeN outtree

Command line options
====================

-h, --help
  Display this message and exit.

-m, --mode=Average|Sum
  Specify mode. (default: Average)
_END
	exit(0);
}
